indexing
	description: "Email Object"
	author: "david s"
	date: "$Date: 2001/07/14 11:25:05 $"
	revision: "$Revision: 1.9 $"

class
	EMAIL

inherit
	MEMORY_RESOURCE

create
	make, make_with_entry

feature -- Initialization

	make is
			-- Initialize the headers table.
		do
			create headers.make (3)
		end

	make_with_entry (header_from, header_to, header_cc, header_bcc, header_subject: STRING) is
			-- Create an email with the 'header_from' and the 'header_to'.
			-- Additions 8/24/03. gh.  Only 1 email address per arg will work. 10/1/03
		require
			needed_info: header_from /= "" -- Void.  Changed 9/30/03 gh.
						 and then header_to /= "" -- Void         Ditto.
		local
			header_date: STRING
			dt: DATE_TIME
			utils: UTILITY
			months,initcap_months: ARRAY [STRING]
			i,start_index: INTEGER
		do	
			create headers.make (3)
			add_header_entry (H_from, header_from)
			add_header_entry (H_to, header_to)
			if header_cc /= "" then add_header_entry (H_cc, header_cc) end  -- "If" (only) added 9/30/03. gh
			if header_bcc /= "" then add_header_entry (H_bcc, header_bcc) end --                       Ditto.
			if header_subject /= "" then add_header_entry (H_subject, header_subject) end -- Line added 8/24/03. gh
			
			-- RFC2822 specifies a "8 Aug 2003 09:39:08 -0600"-type format, while plain dt.out 
			-- format is like "08/08/2003 9:39:08.413 PM", so we use formatted_out.
			!!dt.make_now
            header_date := dt.formatted_out ("dd mmm yyyy [0]hh:[0]mi:[0]ss")
            !!utils.make
            if utils.is_standard_time_in_US (dt) then
            	header_date.append (" -0500") -- East coast specific.
            else
            	header_date.append (" -0400") -- Ditto.
            end

			-- Initcap month name.
			from
				months := <<"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC">>
				initcap_months := <<"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec">>
				i := 1
			until
				header_date.substring_index (months.item (i),1) > 0
			loop
				i := i+1
			end
			start_index := header_date.substring_index (months.item (i),1)
			header_date.replace_substring (initcap_months.item (i), start_index, start_index+2)

			add_header_entry (H_date, header_date)
		end

feature {NONE} -- Basic operations.

	transfer (resource: PROTOCOL_RESOURCE) is
			-- Used when the mailer will receive an email from 'resource'.
		do
			
		end

feature -- Basic operations

	add_header_entry (header_key, header_entry: STRING) is
			-- Add 'header_entry' to header 'header_key',
			-- If no such header exists, create it.
		require
			not_void: header_entry /= Void and then header_key /= Void
		local
			a_header: HEADER
		do
			if headers.has (header_key) then
				a_header:= headers.item (header_key)
				a_header.add_entry (header_entry)
			else
				create a_header.make (header_entry)
				headers.put (a_header, header_key)
				if Default_headers.has (header_key) then
					a_header.enable_contains_addresses
				end
			end
		end

	add_header_entries (header_key: STRING; header_entries: ARRAY [STRING]) is
			-- Add multiple 'header_entries' at once  to 'header_key',
			-- If not such header exists. create it.
			-- No class is calling this feature as yet. 10/01/03 gh
		local
			a_header: HEADER
		do
			if headers.has (header_key) then
				a_header:= headers.item (header_key)
				a_header.add_entries (header_entries)
			else
				create a_header.make_with_entries (header_entries)
				headers.put (a_header, header_key)
				if Default_headers.has (header_key) then
					a_header.enable_contains_addresses
				end
			end		
		end

	remove_header_entry (header_key, header_entry: STRING) is
			-- Remove 'header_entry' from header 'header_key'.
		require
			header_exists: headers.has (header_key)
			header_entry_exists: 
					(headers.item (header_key)). entries.has (header_entry)
		local
			a_header: HEADER			
		do
			a_header:= headers.item (header_key)
			a_header.entries.prune (header_entry)
		ensure
			header_entry_no_longer_exists: 
					not (headers.item (header_key)). entries.has (header_entry)
		end

	remove_header_entries (header_key: STRING) is
		require
			header_exists: headers.has (header_key)
		local
			a_header: HEADER
		do
			a_header:= headers.item (header_key)
			a_header.entries.wipe_out
		end

	has_header_entry (header_key: STRING): BOOLEAN is
		do
			Result:= headers.has (header_key)
		end

	send is
			-- Send email.
		do

		end

	receive is
			-- Receive email.
		do

		end

feature -- Implementation (EMAIL_RESOURCE)

	can_be_received: BOOLEAN is True
		-- Can an email received?

	can_be_sent: BOOLEAN is True
		-- Can an email be send?

end -- class EMAIL


--|----------------------------------------------------------------
--| EiffelNet: library of reusable components for ISE Eiffel.
--| Copyright (C) 1986-2001 Interactive Software Engineering Inc.
--| All rights reserved. Duplication and distribution prohibited.
--| May be used only with ISE Eiffel, under terms of user license. 
--| Contact ISE for any other use.
--|
--| Interactive Software Engineering Inc.
--| ISE Building
--| 360 Storke Road, Goleta, CA 93117 USA
--| Telephone 805-685-1006, Fax 805-685-6869
--| Electronic mail <info@eiffel.com>
--| Customer support: http://support.eiffel.com>
--| For latest info see award-winning pages: http://www.eiffel.com
--|----------------------------------------------------------------

