indexing
	description: "Objects that pull data from db and prep it for display of shopping cart."
	date: "2003/07/02"
	revision: ""
	comment: "Significantly modified copy of ISE's esqlODBC."

class
	PULL_Y_PREP_CART inherit

	RDB_HANDLE

	ACTION
		redefine 
			execute
		end

create

	make

feature -- Public attributes
	p_y_ps_container:  ARRAYED_LIST [DB_RESULT] -- General-purpose record container. Loaded in run_sql. 7/21/03
	
feature {NONE} -- Implementation attributes

	Stringlength: INTEGER is 80	
        base_selection: DB_SELECTION
	session_control: DB_CONTROL
	cart_string: STRING
	utils: UTILITY

feature -- Initialization

	make is
		local
			tmp_string: STRING
		do
			set_data_source("PgDell")
                        login ("pgres1234", "")

			-- Initialization of the Relational Database:
			-- This will set various informations to perform a correct
			-- Connection to the Relational database
			set_base

			-- 'session_control' provides informations control access and 
			--  the status of the database.
			create session_control.make

			-- Start session
			session_control.connect

			if not session_control.is_connected then
					-- Something went wrong, and the connection failed
				session_control.raise_error
				tmp_string.wipe_out
				tmp_string.append ("exit")
			else
				create utils.make -- This class is useful enough to get ready.
				
				-- 'base_selection' provides a SELECT query mechanism.
			    create base_selection.make		

				-- 'cart_string' will hold the data pulled from db.
				create cart_string.make( 300 )   -- This create was nec.
			end -- if not session_control.is_connected
		end
	
feature -- Basics

	terminate_session is
		-- Terminate SQL session (is this nec or is it automatic?). created 6/27/03 gh.
		do
			session_control.disconnect
		end
		
	run_sql (sql_command: STRING) is
			-- Run the SQL command passed in.
		require
			sql_command_exists: sql_command /= Void
		local
			base_update: DB_CHANGE
			log_file: RAW_FILE
		do	
			-- Log the command.  7/22/03
			create log_file.make_open_write ("/tmp/sql_command.log")
			log_file.put_string (sql_command) -- general_store is app-independent (but is platform dep.).
			log_file.close
		
			if is_select_statement (sql_command) then
			-- The query is a SELECT.
			
				-- Set action to be executed after each 'load_result' iteration step.
				-- 'start' and 'execute' method of the current class are to be used.
				base_selection.set_action (Current)

			    base_selection.query (sql_command)                                                
				if session_control.is_ok then
					create p_y_ps_container.make (1) 
					base_selection.set_container (p_y_ps_container)	-- This is this class' general-purpose record container.  
					base_selection.load_result	-- Iterate through resulting data. This also loads the set-to container (previous line).
					base_selection.terminate -- From http://support.eiffel.com/knowledge/eiffelstore/multiple/0001.html. 7/23/03
				else
					manage_errors_and_warnings
				end
				base_selection.terminate
				
			else 
				-- The user updates the database.
				-- 'base_update' provides updating facilities.
				create base_update.make
				base_update.modify (sql_command)
			end -- if is_select_statement...
			manage_errors_and_warnings 
			
			
		end -- run_sql.
		
	end_session is
			-- Terminate session
		do
			session_control.disconnect
		end

feature {NONE} -- Implementation

	manage_errors_and_warnings is
			-- Manage errors and warnings that may have
			-- occurred during last operation.
		do
			if not session_control.is_ok then
				-- There was an error!
				session_control.raise_error
				session_control.reset
				io.new_line
			else
				if session_control.warning_message.count /= 0 then
					io.putstring (session_control.warning_message)
					io.new_line
				end
			end
		end

	read_order is
			-- Get statement from standard input
		do
			io.putstring ("SQL> ")
			io.readline
		end

	seq_string: SEQ_STRING is
		once
			create Result.make (Stringlength)
		end
	
	is_select_statement(st: STRING): BOOLEAN is
			-- Is 'st' a select statement?
		require
			st_not_void: st /= Void
		do
			st.left_adjust 
			-- the above statement is added by Terry Tang
			if st.count /= 0 then

				seq_string.wipe_out
				seq_string.append (st)
				seq_string.to_lower  -- this stmt added by gh 6/10/03
				seq_string.start
				seq_string.search_string_after ("select", 0)
				Result := not seq_string.off
				-- Terryt Tang added the following 8 statements:
				if seq_string.off then
					seq_string.wipe_out
					seq_string.append (st)
					seq_string.to_lower
					seq_string.start
					seq_string.search_string_after ("sql", 0)
					Result := not seq_string.off
					if seq_string.off then
						Result := st.substring(1,1).is_equal("{")
					end
				end
			end
		end -- function is_select_statement.

	execute is
		-- Save shopping cart values in Current.cart_string.
		local
			i: INTEGER
			int_r: INTEGER_REF
			real_r: REAL_REF
			double_r: DOUBLE_REF
			tuple: DB_TUPLE
			any_r: ANY
		do
			from
				i := 1
				create tuple.copy (base_selection.cursor)
				create int_r
				create real_r
				create double_r
			until
				i > tuple.count
			loop -- thru fields
				any_r := tuple.item (i)
				if any_r /= Void then
-- spawn col's: oid     id      cmnt    cdat    cusr    mdat    musr    actv    susr    spwrd   mid     uid   tid
--					if r_any.column_name (i) = "" then
--						
--					end
					if any_r.conforms_to (int_r) then
						int_r ?= any_r  
						cart_string.append (int_r.item.out)
					elseif any_r.conforms_to (real_r) then
						real_r ?= any_r
						cart_string.append (real_r.item.out) 
					elseif any_r.conforms_to (double_r) then
						double_r ?= any_r
						cart_string.append (double_r.item.out)
					else
						cart_string.append (any_r.out)
					end
				else
					cart_string.append ("*void*")
				end
				cart_string.append (" %T")
				i := i + 1
			end -- of fields
		end -- procedure execute.
		
feature -- Access
    
	field (column_name: STRING; tuple: DB_TUPLE): ANY is
		-- Data of `column_name' in given tuple. Created 8/03 gh.
		-- (Any numeric db column will be returned as a DOUBLE_REF, it seems. 9/03)
		require
			cn_not_void: column_name /= Void
			t_not_void: tuple /= Void
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > tuple.count or else equal (tuple.column_name(i), column_name) 
			loop -- thru fields
				i := i + 1
			end -- of fields
			Result := tuple.item (i)
			check column_name_found: i <= tuple.count end
--		ensure
--			column_name_found: i <= tuple.count  -- "Error: local entities may not be used in this context."
	end -- function field.
				
	get_container: like p_y_ps_container is
		-- Return data pulled from the db. Created 6/27/03 gbh.
		do
			Result := p_y_ps_container
		end
		
	get_purchase_data (SID: STRING): STRING is
			-- Will return, eg, "$32.50 to be charged to credit card ending in '4321'". Created 8/10/03 gh.
		require
			not_void: SID /= Void
		local
			uid,ccnum: STRING
			total_cost: DOUBLE_REF
			tuple: DB_TUPLE
			store_file: RAW_FILE
		do
			create tuple.make
			
		    -- Gather up all the shopping cart item data for display, using the SID.  7/10/03                        
			run_sql ("SELECT total FROM sessions WHERE id="+ SID)
			p_y_ps_container.start -- Goes to first record selected, it looks like.  8/18/03
		    tuple.copy (p_y_ps_container.item)  -- Can't use cursor because using container, not db_selection. 7/22/03		    
			total_cost ?= field ("total",tuple)
			
			run_sql ("SELECT uid FROM sessions WHERE id="+ SID)
			p_y_ps_container.start 
		    tuple.copy (p_y_ps_container.item)  
			uid := field ("uid",tuple).out -- .out nec.

			run_sql ("SELECT ccnum FROM u WHERE id="+ uid)
			p_y_ps_container.start 
		    tuple.copy (p_y_ps_container.item)  
			ccnum := field ("ccnum",tuple).out
			if ccnum.count>4 then
				ccnum := ccnum.substring (ccnum.count-3, ccnum.count)				
			end
			Result := "$"+utils.numeric_to_money_string (total_cost)+" to be charged to credit card ending in '"+ccnum+"'"
		
			create store_file.make_open_write ("/tmp/pypc_gpd.txt")
			store_file.put_string (Result) 
			store_file.close
		end -- function get_purchase_data.		
	
	
	get_cart (SID: STRING): STRING is
			-- Return shopping cart as html. Created 7/6/03 gbh.
			-- (Precede all %Ns with a %R to keep the output emailable. 9/13/03)
		require
			sid_not_void: SID /= Void
		local
			tuple: DB_TUPLE
			decimal_place: INTEGER
			query,id,nam,amt,quantity,cost: STRING
			d_ref: DOUBLE_REF
			total_cost,d: DOUBLE
			shipping: REAL
			store_file: RAW_FILE
		do
		    -- Gather up all the shopping cart item data for display, using the SID.  7/10/03                        
			query := "SELECT i.id,i.nam,i.amt, SUM(c.q) as quantity, i.amt*SUM(c.q) as cost FROM cart_items c, sessions s, i %
	                 % WHERE c.sid=s.id AND c.iid=i.id AND s.id=" + SID + " GROUP BY i.id,i.nam,i.amt"
	        run_sql (query)
			from 
				create tuple.make
			    Result := "<table border=1 cellpadding=0>%R%N"+
							 "<thead><tr><td>Item ID</td><td>Description</td><td>Price</td><td>Quantity</td><td>Cost</td></tr>%R%N"+
							 "</thead><tbody>%R%N"
				p_y_ps_container.start
			until
				p_y_ps_container.after
			loop -- For each record:
			    tuple.copy (p_y_ps_container.item)  -- Can't use cursor because using container, not db_selection. 7/22/03
			    id := field ("id",tuple).out
   			    nam := field ("nam",tuple).out
   			    
				-- This section is nec bec, eg, 18.99 in the db becomes 18.989999999999998 either when pulled from the
				-- db or when converted to string. 
				
				d_ref ?= field ("amt",tuple)
				amt := utils.numeric_to_money_string (d_ref) -- Price of one of current item.
				
				d_ref ?= field ("cost",tuple)
				cost := utils.numeric_to_money_string (d_ref)
				d := d_ref.item
				total_cost := total_cost + d -- Adding cost of current item to total_cost.
				
   			    quantity := field ("quantity",tuple).out
   			    
			    Result.append ("<tr><td>"+id+"</td><td>")-- Append the string corresponding to field id. 
			    Result.append (nam+"</td><td>$")		 -- This is the name of the item.

			    Result.append (amt+"</td><td><input  type=text  name=set_quantity_of_iid")
			    Result.append (id)		-- This serves to id item in case quantity update is requested. 7/10/03
			    Result.append (" size=3 maxlength=4 value=")
			    Result.append (quantity+"></td><td>$")-- This serves as default quantity.

			    decimal_place := cost.index_of ('.',1)
			    if decimal_place>0 and decimal_place+2 < cost.count then
			    	cost := cost.substring (1,decimal_place+2)
			    elseif decimal_place>0 and decimal_place+1 = cost.count then
			    	cost.append ("0")
			    end
			    Result.append (cost+"</td></tr>%R%N")	-- Cost is amt * quantity.  
			    
			    p_y_ps_container.forth -- move (1) seemed not to work so trying forth. 7/22/03
			end	-- loop for each record.

			if total_cost <= 10.0 then
				shipping := 5.25
			elseif total_cost <= 25.0 then
				shipping := 6.0
			elseif total_cost <= 50.0 then
				shipping := 6.75
			elseif total_cost <= 75.0 then
				shipping := 7.5
			elseif total_cost <= 100.0 then
				shipping := 8.5
			elseif total_cost <= 125.0 then
				shipping := 9.5
			else
				shipping := 10.5				
			end
			Result.append ("<tr><td colspan=4 align=right><font color=#003399>")
			Result.append ("Shipping &nbsp; </font></td><td>$")
			Result.append (utils.numeric_to_money_string (shipping)+"</td></tr>%R%N")
			total_cost := total_cost + shipping
			Result.append ("<tr><td colspan=4 align=right><font color=#003399>Total &nbsp; </font></td><td>$")
			Result.append (utils.numeric_to_money_string (total_cost)+"</td></tr></tbody></table>%R%N")
			
			-- Save total_cost to sessions.total, for use on confirmation screen. 8/10/03
			run_sql ("UPDATE sessions SET total="+utils.numeric_to_money_string (total_cost)+" WHERE id="+SID)
			
			create store_file.make_open_write ("/tmp/p_y_ps_container.log")
			store_file.put_string (p_y_ps_container.out) 
			store_file.close
		end -- function get_cart.
		
end -- class PULL_Y_PREP_CART.


--|----------------------------------------------------------------
--| EiffelStore: library of reusable components for ISE Eiffel.
--| Copyright (C) 1986-2001 Interactive Software Engineering Inc.
--| All rights reserved. Duplication and distribution prohibited.
--| May be used only with ISE Eiffel, under terms of user license. 
--| Contact ISE for any other use.
--|
--| Interactive Software Engineering Inc.
--| ISE Building
--| 360 Storke Road, Goleta, CA 93117 USA
--| Telephone 805-685-1006, Fax 805-685-6869
--| Electronic mail <info@eiffel.com>
--| Customer support: http://support.eiffel.com>
--| For latest info see award-winning pages: http://www.eiffel.com
--|----------------------------------------------------------------

