indexing
	description	: "This system implements an e-commerce shopping cart."
        date		: "2003/08/7"
	author		: "George Herson"
      
    
class
	ROOT_CLASS

inherit
	CGI_INTERFACE 
	STORABLE  -- Added 7/15/03 to store form_data (from browser request).
	
creation
	make
	
feature -- Initialization

    debug_mode: BOOLEAN is true -- Lest only "Internal Server Error" is shown for a run-time error. gh 8/7/03
    	-- A deferred feature of CGI_INTERFACE.
    		
    http_mode: BOOLEAN	
    	-- May be set to true in execute, below. 7/22/03  
    	
    cookies_from_file: HASH_TABLE[STRING,STRING]
		-- Set by the current.retrieve_cookies_From_file procedure. gh 8/17/03
		-- (This extra feature is nec to preserve CQS. (And it wasn't nec w/form_data bec I could re-use form_data 
		-- when http_mode is false.)
    	
feature {NONE} -- Operations

	email_receipt (subject, body: STRING) is
		local
			email: EMAIL
		do
			-- Build email (Angle brackets will be added to addresses. gh 9/1/03)

			!!email.make_with_entry ("from@merchant.com",customer_email,"cc@merchant.com","bcc@merchant.com",subject) -- works

			email.set_message ("%R%N<html><body>%R%N%
			%<p/>Dear Customer:%R%N%
			%<p/>Thanks for your purchase!%R%N%
			%<p/>If items are not in stock, it normally takes a couple days for us to get them.  All in stock %
			%items will be shipped within 24 hours of order.  You'll be notified via email as soon as %
			%your items are shipped.%R%N"+body+"</body></html>") -- Add tags not added by pypc.get_cart.%
			send_email (email)
		end
		
	send_email (email: EMAIL) is
		local
			send: SMTP_PROTOCOL		
		do
			!!send.make ("myhostname.datainformation.com","username") 
			send.initiate_protocol
			send.transfer (email)
			send.close_protocol
		end
		
	pull_info_from_last_http_request is
		-- Fill in form_data and Cookies_from_file. gh 8/17/03
		-- (Based on a loop in directory_selection_dialog_windows.e's insert_in_directory_list. gh 8/17/03)
		local
			the_temp_dir,i_file: STRING
			dir: DIRECTORY
			file,newest_form_data_file,newest_cookies_file: RAW_FILE
		do
			from
				the_temp_dir := "/DEC/tmp/"
				create dir.make_open_read (the_temp_dir)
				create newest_form_data_file.make (".")
				create newest_cookies_file.make (".")
				dir.start
				dir.readentry
				i_file := dir.lastentry
			until
				i_file = Void
			loop
				if not i_file.is_equal (".") and not i_file.is_equal ("..") then
					-- See if i_file is the latest form_data or latest cookies data file.
					if i_file.substring_index ("form_data",1)=1 then
						create file.make_open_read (the_temp_dir+i_file) -- Shadow of actual file, that lets us access info about it.
						if equal (newest_form_data_file.name,".") or else file.date > newest_form_data_file.date then
							newest_form_data_file := file
						end	
					elseif i_file.substring_index ("cookies",1)=1 then
						create file.make_open_read (the_temp_dir+i_file) 
						if equal (newest_cookies_file.name,".") or else file.date > newest_cookies_file.date then
							newest_cookies_file := file
						end	
					end
				end
				-- Move to next file.
				dir.readentry
				i_file := dir.lastentry    
			end -- end loop.
			dir.close -- All files searched.
				
			if newest_form_data_file.name="." then
				raise_error ("No form_data file found. %N")
			elseif newest_cookies_file.name="." then
				raise_error ("No cookies file found. %N")
			else
				
				form_data_From_file (newest_form_data_file) 
				retrieve_cookies_From_file (newest_cookies_file)	
			end 

		end

feature -- Execution
		
	execute is                   
			-- A deferred feature of CGI_INTERFACE.
			
		local
			htmlfile: HTML_PAGE
			page,s,show_this_html: STRING					
		do	        	
			if form_data.count = 0 then
				-- We're debugging (vs http mode). Pull info received during user's 
				-- last web access. 7/22/03
				pull_info_from_last_http_request
				page:="/DEC/home/httpd/html/shopping_demo/" 
			else 
				page:="/home/httpd/html/shopping_demo/" -- (page is a string that holds the filespec).
				http_mode := TRUE
				-- Save the form_data for a possible debugging run later.  7/22/03
				s := "form_data"+util.random_file_name
				form_data_To_file ("/tmp/" + s + ".out") -- ("nobody" needs write perms to this file.)
			end

            -- Det desired act and show the assoc'ed page so act can be carried out (on this or the next http request).
            response_header.generate_text_header -- Always nec 1st step w/the CRH class, and nec before SID call. (Inherited attribute)
            show_this_html := "" -- ({HTML_PAGE}.replace_marker requires this arg to be non-void. 9/14/03)
            if field_defined ("act") then
	            if equal ("add", text_field_value ("act")) then 
			            -- Add to cart. Done now.
	            	Current.add_to_cart (text_field_value ("iid")) -- ("iid" match is case sensitive.)
           	        page.append_string ("cart.html")
		            show_this_html := sql.get_cart (sid) -- SELECT from db and generate the cart html.  7/10/03
		            
	            elseif equal ("quant_upd", text_field_value ("act")) then 
	            		-- Update quantities of items in cart per user request.  Done now.
	            	Current.update_quantities
	            	page.append_string ("cart.html") 
		            show_this_html := sql.get_cart (sid) -- Gen the cart html via a sql select.  7/10/03
		            
	            elseif equal ("checkout", text_field_value ("act")) then
	            		-- Present page that allows checkout.
					page.append_string ("checkout.html")
					
	            elseif equal ("confirm", text_field_value ("act")) then
	            		-- Authenticate (or insert new) user. If that goes smoothly, present purchase confirmation page.
	            	Current.process_user
	            	if is_login_valid then
						page.append_string ("confirm.html")
						show_this_html := sql.get_purchase_data (sid) 
							-- The returned string will, in the below, substitute for placeholder.
	            	else
	            		page.append_string ("invalid_login.html")
	            	end
								-- (Copy the files in /win/temp/pp/act=buy into $DEC/tmp/ to test the below. 9/14/03)
				elseif equal ("buy", text_field_value ("act")) then
	            	Current.finalize_purchase -- Update db to reflect purchase.
--					delete_cvv2 (sid or tid) --todo
					show_this_html := sql.get_cart (sid) -- Gen the cart html via a sql select.
					email_receipt ("Renaissance Receipt #"+tid, show_this_html) 
						-- Email the customer, cc: Dev and bcc: gh.
					page.append_string ("thankyou.html")     -- Thank customer.
	            else
	            	raise_error ("Unrecognized act requested. %N")
	            end
			else         	
				raise_error ("No act requested. %N")
			end
           	
            -- Prepare an HTML page for output from a "template" (html file that may contain symbols/words
            -- for replace()'ing).
            !!htmlfile.make_from_template (page)  
           	htmlfile.replace_marker("<!--placeholder-->", show_this_html) 
			response_header.send_to_browser  			-- Send any accumulated headers, e.g., a cookie.				
			-- io.putstring( htmlfile.out )    
			response_header.Output.put_string (htmlfile.out)
			sql.end_session
	end -- procedure execute.
		
	finalize_purchase is
			-- update db to note the purchase was finalized, with $ amount. 
			-- created 9/9/03 gh
		local
			record_container: ARRAYED_LIST [DB_RESULT]
			tuple: DB_TUPLE
			total_purchase_amt: DOUBLE_REF
		do
			-- This section is to get the total purchase amount.
			sql.run_sql ("SELECT sessions.total FROM i,cart_items,sessions WHERE %
			             %i.id=cart_items.iid AND cart_items.sid=sessions.id AND cart_items.sid="+SID)
           	create record_container.make (1)
			record_container := sql.get_container
			if equal (0,record_container.count) then
				raise_error ("Error: Select in finalize_purchase found 0 rows. %N")	
			end
			record_container.start -- Go to first record selected.  8/18/03
		   	!!tuple.copy (record_container.item)  -- Can't use cursor because using container, not db_selection. 7/22/03		    
			total_purchase_amt ?= sql.field ("total",tuple)     

			if not http_mode then -- We're debugging, so we need to see if the t record already exists. 9/29/03
				sql.run_sql ("SELECT * FROM t WHERE sid="+SID)
				if sql.p_y_ps_container.count = 0 then
					sql.run_sql ("INSERT INTO t (sid,amt) VALUES ("+SID+","+total_purchase_amt.out+")")
				end
			else
				sql.run_sql ("INSERT INTO t (sid,amt) VALUES ("+SID+","+total_purchase_amt.out+")") -- cclast4, resp, respsum are the last 3 cols.
			end
			sql.run_sql ("UPDATE sessions SET stat='t' WHERE id="+SID) -- Mark session as having ended in transaction.
			
			-- Delete cookie to log out user.  (So that sid won't be re-used for another purchase
			-- and i can use it as the order id (and constrain t.sid to uniqueness)). 9/03
			response_header.set_cookie ("SID","",Void,"/",Void,Void) 
		end	
		
	update_quantities is
			-- Update quantities of items in cart per user request.  7/11/03
		local
			http_var_name,new_quantity,iid,query: STRING
		do	
            	-- Loop thru all form fields posted.  Those that start with "set...", update the iid
            	-- that follows with the associated value. 
            	from 
            		Current.form_data.start
            	until
            		form_data.off
            	loop
            		if equal ("set_quantity_of_iid", form_data.key_for_iteration.substring (1,19)) then -- (using "=" didn't work. 7/21/03)
            			-- The current item is for quantity update.  7/11/03
            			
            			http_var_name := form_data.key_for_iteration -- 'set_...' + iid
            			new_quantity := form_data.item_for_iteration.item.out
            			iid := http_var_name.substring (20,http_var_name.count) -- Strip away leading 'set_...'
            			query := "DELETE FROM cart_items WHERE sid="+SID+" AND iid="+iid
		            	sql.run_sql (query)
		            	if not equal ("0",new_quantity) then
			            	sql.run_sql ("INSERT INTO cart_items (sid,iid,q) VALUES ("+SID+","+iid+","+new_quantity+")")
			            end
            		end
   			        form_data.forth
            	end            	
		end -- command update_quantities.            
		
	add_to_cart (iid: STRING) is
			-- User clicked on "Add to cart"; insert a new row for item in carts table. 7/10/03
		require
			iid /= Void
		local
			query: STRING	
		do          	            	
			query := "INSERT INTO cart_items (sid,iid,q) VALUES ("+SID+","+iid+",1)"  
				-- Quantity "1" works because the total will be SUMmed.  7/10/03
            sql.run_sql (query)
		end -- add_to_cart.		
		
	process_user is
			-- Validate user, enter/update user info, or simply note that log in info does not match
			-- that in db.  10/13/03. 
		local
			email,pwrd,login_attempt,fnam,mnam,lnam,phone,addr,city,state,zip,cctype,ccnum,cvv2: STRING
			ccexpmo,ccexpyr,uid,query,db_pwrd,reg_or_user_update_attempt: STRING
			tuple: DB_TUPLE
			record_container: ARRAYED_LIST [DB_RESULT]
			is_email_address_extant,is_reg_or_user_update_valid: BOOLEAN
		do	
            	-- Loop thru all form fields posted to capture their data in local vars. 
            	from 
            		Current.form_data.start
            	until
            		form_data.off
            	loop
            		if equal ("email", form_data.key_for_iteration) then -- 
            			email := form_data.item_for_iteration.item.out
		     		elseif equal ("pwrd", form_data.key_for_iteration) then --
            			pwrd := form_data.item_for_iteration.item.out
            		elseif equal ("login_attempt", form_data.key_for_iteration) then --	
           				login_attempt := form_data.item_for_iteration.item.out
            		elseif equal ("reg_or_user_update_attempt", form_data.key_for_iteration) then --	
           				reg_or_user_update_attempt := form_data.item_for_iteration.item.out
            		elseif equal ("fnam", form_data.key_for_iteration) then --	
           				fnam := form_data.item_for_iteration.item.out
            		elseif equal ("mnam", form_data.key_for_iteration) then --	
           				mnam := form_data.item_for_iteration.item.out
           			elseif equal ("lnam", form_data.key_for_iteration) then --	
           				lnam := form_data.item_for_iteration.item.out
            		elseif equal ("phone", form_data.key_for_iteration) then --	
           				phone := form_data.item_for_iteration.item.out
            		elseif equal ("addr", form_data.key_for_iteration) then --	
           				addr := form_data.item_for_iteration.item.out
           			elseif equal ("city", form_data.key_for_iteration) then --	
           				city := form_data.item_for_iteration.item.out
           	 		elseif equal ("state", form_data.key_for_iteration) then --	
           				state := form_data.item_for_iteration.item.out
           	  		elseif equal ("zip", form_data.key_for_iteration) then --	
           				zip := form_data.item_for_iteration.item.out           				
              		elseif equal ("cctype", form_data.key_for_iteration) then --	
           				cctype := form_data.item_for_iteration.item.out
            		elseif equal ("ccnum", form_data.key_for_iteration) then --	
           				ccnum := form_data.item_for_iteration.item.out
            		elseif equal ("cvv2", form_data.key_for_iteration) then --	
           				cvv2 := form_data.item_for_iteration.item.out
           			elseif equal ("ccexpmo", form_data.key_for_iteration) then --	
           				ccexpmo := form_data.item_for_iteration.item.out
           			elseif equal ("ccexpyr", form_data.key_for_iteration) then --	
           				ccexpyr := form_data.item_for_iteration.item.out
            		end
   			        form_data.forth
            	end            	
            	
            	-- Whether the user is logging in or registering, we need to det if the entered email
            	-- address pre-exists in the database.
            	sql.run_sql ("SELECT id,pwrd FROM u WHERE email='"+email+"'") -- Form's JS ensures email entry exists.
           		create record_container.make (1)
				record_container := sql.get_container
				if record_container.count = 0 then
					is_email_address_extant := False
					is_login_valid := False 	-- This is a class attribute.
				else
					record_container.start -- Go to first record selected.  8/18/03
			    	!!tuple.copy (record_container.item)  -- (Can't use cursor because using container, not db_selection. 7/22/03)
					uid := sql.field ("id",tuple).out
					db_pwrd := sql.field ("pwrd",tuple).out
					is_email_address_extant := True
					if equal (db_pwrd, pwrd) then
						is_login_valid := True
					else
						is_login_valid := False
					end
				end

            	if reg_or_user_update_attempt /= Void then 
            		-- User is attempting to register or update their info, not simply log in.
            		if not is_email_address_extant then
		            	-- Email address is new, so simply insert user info into db.
	            		query := "INSERT INTO u (email,pwrd,fnam,mnam,lnam,phone,addr,city,state,zip,cctype,ccnum,cvv2,ccexpmo,ccexpyr) VALUES ('"
						query.append (email+"','"+pwrd+"','"+fnam+"','"+mnam+"','"+lnam+"','"+phone+"','"+addr+"','"+city+"','"+state) 
						query.append ("','"+zip+"','"+cctype+"','"+ccnum+"','"+cvv2+"',"+ccexpmo+","+ccexpyr+")") -- The last 2 are of type int2.
						sql.run_sql (query)	
						
						is_reg_or_user_update_valid := True
						
						-- Get the uid just created.
						sql.run_sql ("SELECT id FROM u WHERE email='"+email+"'") 
		           		create record_container.make (1)
						record_container := sql.get_container
						record_container.start -- Go to first record selected.  8/18/03
				    	!!tuple.copy (record_container.item)  -- (Can't use cursor because using container, not db_selection. 7/22/03)
						uid := sql.field ("id",tuple).out
					end
					if is_login_valid then 
						-- Email address and password match db's, so update user info in db.
						sql.run_sql ("DELETE FROM u WHERE id="+ uid)
						query := "INSERT INTO u (id,email,pwrd,fnam,mnam,lnam,phone,addr,city,state,zip,cctype,ccnum,cvv2,ccexpmo,ccexpyr) VALUES ("
						query.append (uid+",'"+email+"','"+pwrd+"','"+fnam+"','"+mnam+"','"+lnam+"','"+phone+"','"+addr+"','"+city+"','"+state) 
						query.append ("','"+zip+"','"+cctype+"','"+ccnum+"','"+cvv2+"',"+ccexpmo+","+ccexpyr+")") -- The last 2 are of type int2.
						sql.run_sql (query)	
						
						is_reg_or_user_update_valid := True
					end
            	end	
            	
            	if is_login_valid or is_reg_or_user_update_valid then
					sql.run_sql ("UPDATE sessions SET uid="+ uid +" WHERE id="+ SID)
				end
		end -- command process_user.   
		
feature -- Status report

	is_login_valid: BOOLEAN
			-- Set in process_user and used in execute where act=confirm.
		
feature {NONE} -- Implementation

    tid: STRING is 
    		-- Transaction id. created 9/03
    	local
    		tuple: DB_TUPLE
			record_container: ARRAYED_LIST [DB_RESULT]
    	once
    		sql.run_sql ("SELECT t.id FROM t,sessions WHERE sessions.id=t.sid AND sessions.id="+SID)
    		
           	create record_container.make (1)
			record_container := sql.get_container
			if equal (0,record_container.count) then
				raise_error ("Error: transaction id not found. %R%N")	
			end
			record_container.start -- Go to first record selected.  8/18/03
		   	!!tuple.copy (record_container.item)  -- Can't use cursor because using container, not db_selection. 7/22/03		    
			Result ?= sql.field ("id",tuple).out     
    	end

    customer_email: STRING is
    		-- Customer's email address. created 9/14/03
    	local
    		tuple: DB_TUPLE
			record_container: ARRAYED_LIST [DB_RESULT]
    	once
    		sql.run_sql ("SELECT u.email FROM u,sessions WHERE sessions.uid=u.id AND sessions.id="+SID)
    		
           	create record_container.make (1)
			record_container := sql.get_container
			if equal (0,record_container.count) then
				raise_error ("Error: customer_email found 0 rows. %R%N")	
			end
			record_container.start -- Go to first record selected.  8/18/03
		   	!!tuple.copy (record_container.item)  
			Result ?= sql.field ("email",tuple)     
    	end
    
    sql: PULL_Y_PREP_CART is
    		-- Customized code for pulling and prepping shopping cart.
    	once
    		!!Result.make -- Used for all sql commands in this class.  7/12/03	
		end
		
	util: UTILITY is
			-- Miscellaneous tools.
		once
			!!Result.make -- Accessible to all routines in this class.
		end
		
		
	form_data_To_file (file_name: STRING) is
			-- Write form_data to file_name. 7/16/03 
		local
			store_file: RAW_FILE
		do
			create store_file.make_open_write (file_name)
			store_file.general_store (form_data) -- general_store is app-independent (but is platform dep.).
			store_file.close
		end
		
	cookies_To_file (file_name: STRING) is
			-- Write Cookies to file_name. 7/16/03 
		local
			store_file: RAW_FILE
		do
			create store_file.make_open_write (file_name)
			store_file.general_store (Cookies) 
			store_file.close
		end
		
	form_data_From_file (file_name: FILE) is
			-- Get form_data from file_name. 7/16/03 gh
		local
			get_file: RAW_FILE
			s: STRING
		do
			s := file_name.name
			create get_file.make_open_read (s)
			form_data ?= get_file.retrieved 
			get_file.close
		end

	retrieve_cookies_From_file (file_name: FILE) is
			-- Get cookies from file_name. 7/16/03 gh
		local
			get_file: RAW_FILE
			s: STRING
		do
			s := file_name.name
			create get_file.make_open_read (s)
			cookies_from_file ?= get_file.retrieved 
			get_file.close
		end

	SID: STRING is 
			-- Calculate session ID and store it to cookie and db. 7/11/03
		local
			i: INTEGER
			query,s: STRING
			record_container: ARRAYED_LIST [DB_RESULT]		
		once	
			if not http_mode then 	
				-- The Cookie hash_table comes from the environment, not form_data, so another hash table is created
				-- when we're debugging, called "cookies_from_file", that reads the last, server-saved cookies. 8/17/03
				if cookies_from_file.has ("SID") then  
					-- Session pre-exists: grab SID. 
					Result := Current.cookies_from_file.item ("SID") 
				else  
			  		Result := "1230910134"
			  	end
			else	
				-- Save the Cookies info.  8/17/03
				s := "cookies"+util.random_file_name
				cookies_To_file ("/tmp/" + s + ".out") -- "nobody" needs write perms to this file.	
				
				util.string_To_file (cookies.out, "/tmp/out_cookies")  -- Output gross cookie info for visual inspection.
				-- Now output cookies' keys to file, also for visual inspection.
				from 
					s := ""
					i := 1
				until
					i > cookies.count
				loop
					s.append (cookies.current_keys.item (i)+";")
					i := i+1
				end
				util.string_To_file (s, "/tmp/outcookies.keys")  -- 7/23/03

				if cookies.has ("SID") and then cookies.item("SID").count>0 then  
					-- Session pre-exists: grab SID. 
					Result := Current.cookies.item ("SID") 
				else  
					-- Gen a SID and store it to cookie.
					Result := util.gen_random_number 
					response_header.set_cookie ("SID",Result,Void,"/",Void,Void) 
				end
          	end -- if http_mode.

			-- In db, record SID if new or the current time if not. We start by det'ing pre-existance.
			if not http_mode then 	-- We're debugging, so we don't care what stat is (eg, the xaction is already complete and stat=t).
				sql.run_sql ("SELECT * FROM sessions WHERE id="+Result)
			else
				sql.run_sql ("SELECT * FROM sessions WHERE stat IS Null AND id="+Result)
			end
			create record_container.make (1)
			record_container := sql.get_container
			if (record_container.count > 0) then
				-- This session pre-exists.  Update last_access in sessions table. 7/10/03
			
				query := "UPDATE sessions set last_access = timestamp('now') WHERE id="+Result
				sql.run_sql (query)						
            else
   				-- This session is new; insert SID etc into sessions table.
   			
--				sessions' values, in order: $SID,NOW(),$HTTP_REFERER,$IP_ADDRESS,$MID.
				query := "INSERT INTO sessions VALUES ("+Result
				query.append (",timestamp('now'),'pretend','192.168.1.100',2,null)") --todo Use real values.
					-- Didn't work with double quotes and ::text.
--				s.append (",'2003-07-06 18:02:26-05','testing','192.168.1.100',2)") -- Works.
				sql.run_sql (query)						
            end

		end	-- function SID.
	
end -- class ROOT_CLASS
