indexing
	description: "Objects that ..."
	author: ""
	date: "$Date: 2001/07/14 11:25:05 $"
	revision: "$Revision: 1.13 $"

class
	SMTP_PROTOCOL

inherit
	SENDING_PROTOCOL

create
	make

feature -- Initialization

	make (host:STRING; user: STRING) is
			-- Create an smtp protocol with 'host, 'user' and default port.
		do
			hostname:= host
			username:= user
			port:= default_port
		end

feature -- Access

	smtp_code_number: INTEGER
		-- Code number of the reply

	smtp_reply: STRING
		-- Replied message from SMTP protocol

	pipelining: BOOLEAN
		-- Does the connection use pipeline?

feature -- Setting

	enable_pipelining is
			-- Set pipelining.
		do
			pipelining:= True
		end

	disable_pipelining is
			-- Unset pipelining.
		do
			pipelining:= False
		end

feature -- Access EMAIL_PROTOCOL

	default_port: INTEGER is 25
		-- Smtp default port

feature -- Implementation (EMAIL_RESOURCE).

	transfer (resource: MEMORY_RESOURCE) is
			-- Send the email and add a %R%N at the end of the message. 
		do
			if not error then
				memory_resource := resource
				recipients:= Void
				send_mail
			end
--			disable_transfer_error
		end

feature -- Basic operations.

	initiate_protocol is
			-- Initiate the smtp connection
			-- It can be a helo or a ehlo connection.
		do
			connect
			if pipelining then
				send_command (ehlo + username, Ok)
			else
				send_command (helo + username, Ok)
			end
			if not error then
				enable_initiated
			end
		end

	close_protocol is
			-- Terminate the connection.
		require else
			connection_exists: is_connected
		do
			send_command (Quit, Ack_end_connection)
--			if not error then
				socket.cleanup
				disable_initiated
				disable_connected
				disable_transfer_error
				set_transfer_error_message ("")
--			end
		end

feature -- Implementation (EMAIL_RESOURCE)

	can_receive: BOOLEAN is False
		-- Can the Smtp protocolreceive?

	initialize is
			-- Initialize the protocol to send a new email.
		do

		end

feature {NONE} -- Implementation

	username: STRING
		-- Smtp user name (Needed to initiate the connection).


feature {NONE} -- Basic operations

	decode (s: STRING): INTEGER is
			-- Retrieve the number corresponding to the message receive from the smtp server.
		local
			s_int: STRING
		do
			smtp_reply:= s
			s_int:= s.substring (1, 4)
			Result:= s_int.to_integer
		end

	send_command (s: STRING; expected_code: INTEGER) is
			-- send a string 's' to the server and result the code response.
		local
			response: STRING
			util: UTILITY
			cgi_error_handler: CGI_ERROR_HANDLING
		do
			!!util.make
			util.string_To_file (s + "%R%N","/tmp/sent2smtpserver.txt")  -- debuggery 9/13/03 gh
			socket.put_string (s + "%R%N")
			socket.read_line  -- When smtp server unresponsive, read_line will loop forever with calls to {STRING}.extend. gh
			response:= socket.last_string
			smtp_code_number := decode (response)
			if (smtp_code_number /= expected_code) then
				enable_transfer_error
				set_transfer_error_message (smtp_reply)
					-- The above 2 lines effectively do nothing, so i added the below 2 lines 9/13/03. gh
				!!cgi_error_handler
				cgi_error_handler.raise_error ("Error: "+smtp_reply+" is not "+expected_code.out+", the expected SMTP reply.%R%N") 
			end
		end

feature {NONE} -- Basic operations

	send_mail is
			-- Send mail using smtp protocol.
		do
			send_mails
			if memory_resource.headers.has (H_bcc) then
				enable_bcc_mode
				send_mails
				disable_bcc_mode
			end
		end

	send_mails	is
		do
				header_from:= memory_resource.header (H_from).unique_entry
				sub_header:= ""
				set_recipients
				build_sub_header
				sub_header.append ("MIME-Version: 1.0%R%N") -- Added 9/30/03 gh. This worked to make the html an attachment in snet mail.
				sub_header.append ("Content-Type: text/html; charset=US-ASCII%R%N") -- Added 9/03 gh
				sub_header.append ("%R%N") -- (The previous 2 chars are also %R%N.) RFC2822 calls for a blank line to separate header from body.  Added 9/1/03 gh
				send_all
		end

	set_recipients is
			-- Fill 'recipients' to know who will receive the resource,
			-- and fill 'header_from'
		require
			header_to_exists: memory_resource.headers.has (H_to)
		local
			a_header: HEADER
		do
			if not bcc_mode then -- Use "to" list and append "cc" list, else use "bcc" list. gh 9/03
				a_header:= memory_resource.header (H_to)
				recipients:= clone (a_header.entries)
				a_header:= memory_resource.header (H_cc)
				if a_header /= Void then
					from 
						a_header.entries.start
					until
						a_header.entries.after
					loop
						recipients.extend (a_header.entries.item)
						a_header.entries.forth
					end
				end
			else
				recipients:= memory_resource.header (H_bcc).entries
			end
		end

	build_sub_header is
			-- Build the header of the message in 'sub_header'.
		do
			from 
				memory_resource.headers.start
			until
				memory_resource.headers.after
			loop
				add_sub_header (memory_resource.headers.key_for_iteration)
				memory_resource.headers.forth
			end
		end

	add_sub_header (sub_header_key: STRING) is
			-- Add new header line 'sub_header_key',
			-- A distinction is done in case the mail is bcc or not.
	-- I added the "%R"s below on the advice of a qmail error re: bare LFs. gh 8/21/03
		require
			key_exists: memory_resource.headers.has (sub_header_key)
			sub_header_exists: sub_header /= Void
		local
			a_header: HEADER
		do
			a_header:= memory_resource.header (sub_header_key)
			from 
				a_header.entries.start
			until
				a_header.entries.after
			loop
				
				if not bcc_mode and sub_header_key.is_equal (H_bcc) then
						-- Do nothing. This recipient won't appear in any header but will still get a copy of the email (via a RCPT TO: smtp command). gh 9/29/03
				else
					sub_header.append (sub_header_key + ":" + a_header.entries.item + "%R%N")
				end
		-- Added above section (~5 lines) to replace below 11 lines on 9/29/03 gh.				
--				if bcc_mode then	-- I think this If is to format the message with "to:" for the bcc recip's,
--				                    -- and leave out the other recipients when sending these bcc's.  Unnec. gh
--					if sub_header_key.is_equal (H_bcc) then
--						sub_header.append (H_to + ":" +a_header.entries.item + "%R%N")
--					end
--					if not (sub_header_key.is_equal (H_to) or sub_header_key.is_equal (H_cc)) then
--						sub_header.append (sub_header_key + ":" + a_header.entries.item + "%R%N")
--					end
--				else
--					sub_header.append (sub_header_key + ":" + a_header.entries.item + "%R%N")
--				end

				a_header.entries.forth
			end
		end

	send_all is
		-- Send the mail considering the correct information.
		-- I replaced all "if not error"s in this routine with "check..." to be notified of problems. gh 9/4/03
		do
			mail_message:= clone (memory_resource.mail_message)
			mail_signature:= memory_resource.mail_signature

			send_command (Mail_from + "<" + header_from + ">", Ok) -- (The "Mail_" identifiers correspond 
			-- to SMTP commands as opposed to message headers (which are largely window dressing). The 
			-- headers are all sent after the "DATA" command. gh 9/03)
			check not error end -- if not error then 
				from 
					recipients.start
				until
					recipients.after
				loop
					check not error end -- if not error then
						send_command (Mail_to + "<" + recipients.item + ">", Ok)
					-- end
					recipients.forth
				end

				check not error end -- if not error then
					send_command (Data, Data_code) -- Send "DATA", and expect 354 in return. gh
					mail_message.prepend (sub_header) -- Prepend header lines to message body.
					if mail_signature /= Void then
						mail_message.append ("%R%N" + mail_signature) -- I added the %R 9/1/03. gh
					end
					mail_message.replace_substring_all ("%N.", "%N..") 
					-- mail_message.append ("%N.%N") -- This line changed to
					mail_message.append ("%R%N.") -- this line per http://groups.yahoo.com/group/ise_users/message/1776. gh 8/19/03
					-- Why is above line nec? When commented, we hung at send_command's 'socket.put_string (s + "%R%N")' (which calls NETWORK_STREAM_SOCK's read_line). gh 9/1/03
					check not error end -- if not error then
						send_command (mail_message, Ok)
					-- end
				-- end
			-- end
		end


feature {NONE} -- Access

	recipients: ARRAYED_LIST [STRING]
		-- Header to use with the command 'Mail_to'

	header_from: STRING
		-- Header to use with the command 'Mail_from'.

	sub_header: STRING
		-- Sub_header: data before the message.

	mail_message: STRING
		-- Message.

	mail_signature: STRING
		-- Signature.

	bcc_mode: BOOLEAN
		-- Is bcc mode activated?

feature {NONE} -- Implementation

	enable_bcc_mode is
		do
			bcc_mode:= True
		end

	disable_bcc_mode is
		do
			bcc_mode:= False
		end

end -- class SMTP_PROTOCOL


--|----------------------------------------------------------------
--| EiffelNet: library of reusable components for ISE Eiffel.
--| Copyright (C) 1986-2001 Interactive Software Engineering Inc.
--| All rights reserved. Duplication and distribution prohibited.
--| May be used only with ISE Eiffel, under terms of user license. 
--| Contact ISE for any other use.
--|
--| Interactive Software Engineering Inc.
--| ISE Building
--| 360 Storke Road, Goleta, CA 93117 USA
--| Telephone 805-685-1006, Fax 805-685-6869
--| Electronic mail <info@eiffel.com>
--| Customer support: http://support.eiffel.com>
--| For latest info see award-winning pages: http://www.eiffel.com
--|----------------------------------------------------------------

