indexing
	description: "Objects that are useful for a variety of classes."
	author: "George Herson"
	date: "8/30/03"

class
	UTILITY inherit
		
	EXCEPTIONS -- not yet used in favor of CGI_ERROR_HANDLING. 9/30/03

create
	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'.
		do
			
		end

feature -- Access

feature -- Measurement

	is_standard_time_in_US (given_dt: DATE_TIME): BOOLEAN is
		-- "DST ... is based on the continental US system, which has transitions on the first Sunday in 
		-- April and the last Sunday in October." -- ftp://time-b.timefreq.bldrdoc.gov/pub/daytime/nistime.man 1/3/01. 
		-- created 8/30/03 gh
		require
			given_dt /= Void
		local
			DST_start,DST_end: DATE_TIME	
			given_year,i: INTEGER
		do
			given_year := given_dt.year

			-- Find DST (Daylight Saving Time) start:
			from 
				i:=1 
				!!DST_start.make (given_year,4,i,0,0,1) -- 1st day of April.
			until DST_start.date.day_of_the_week = 1 -- Until Sunday is found (1 stands for Sunday).
			loop 
				i:=i+1
				!!DST_start.make (given_year,4,i,0,0,1) -- Next day of April.
			end
			!!DST_start.make (given_year,4,i,2,0,0) -- 2am on April's first Sunday.

			-- Find DST end:
			from 
				i:=31
				!!DST_end.make (given_year,10,i,0,0,1) -- Last day of October.
			until DST_end.date.day_of_the_week = 1 -- Until Sunday is found.
			loop 
				i:=i-1
				!!DST_end.make (given_year,10,i,0,0,1) -- Previous day of October.
			end
			!!DST_end.make (given_year,10,i,2,0,0) -- 2am on October's last Sunday.
			
			-- If the given (U.S.) date-time is earlier than 2am on April's first Sunday, etc.,
			-- standard time is in effect, else, not.
			if given_dt < DST_start or given_dt >= DST_end then
				Result := True
			else
				Result := False
			end 
		end
		
feature -- Status report

feature -- Status setting

feature -- Cursor movement

feature -- Element change

feature -- Removal

feature -- Resizing

feature -- Transformation

feature -- Conversion

	numeric_to_money_string (a_numeric: NUMERIC): STRING is
		-- Take a REAL or DOUBLE, round to 2 decimal places, and return as string. gh 8/7/03
		require
			not_void: a_numeric /= Void
		local
			real_r: REAL_REF
			r: REAL
			double_r: DOUBLE_REF
			d: DOUBLE
			cgi_error_handler: CGI_ERROR_HANDLING
		do
			create real_r
			create double_r
			if a_numeric.conforms_to (real_r) then
				real_r ?= a_numeric
				r := real_r.item
				Result := (r * 100.0).rounded.out
--					amt := amt.substring (1,decimal_place+2)
			elseif a_numeric.conforms_to (double_r) then 
				double_r ?= a_numeric
				d := double_r.item
				Result := (d * 100.0).rounded.out
			else
				!!cgi_error_handler
				cgi_error_handler.raise_error ("Error: Type of actual argument to {UTILITY}.numeric_to_money_string is unrecognized.")
			end
			Result := Result.substring (1,(Result.count-2)) +"."+  Result.substring ((Result.count-1),Result.count)
		end

feature -- Duplication

feature -- Miscellaneous

feature -- Randomness

	gen_random_number: STRING is
	    -- Generates a random integer in the range 0 to 2,147,483,647 w/o need of an external seed.  
	    -- The number generated has no relationship to the last random integer generated. gh 4/3/03.  
	                    -- (2^31 -1 = 2,147,483,647)
    	local
            seed: INTEGER
	    fineseconds: DOUBLE
            temptime: TIME
            randomizer: RANDOM
        do
	    !!temptime.make_now
            fineseconds := temptime.fine_seconds -- fineseconds is the #.## of seconds including fractional second (2 sig figs)) 
            -- that have elapsed since midnight. 60s x 60m x 24h = 86400.00s 
            -- Multiply fineseconds by 100 and discard the remaining fraction.
            seed := (fineseconds * 100).truncated_to_integer + 1049943 -- The addend (an arbitrary #) is to make the seed harder to guess.
            -- 8,640,000 + 1,049,943 = 9,689,943 is max seed (any seed <2^31 is legal).
            create randomizer.make
            Result := randomizer.next_random( seed ).out
  	ensure
    	    not_void: Result /= Void
        end -- function gen_random_number.                         


	random_file_name: STRING is
		-- Return a unique file name. From ISE_Users msg 1487. 7/21/03
	local
		da: DATE_TIME
	do
		Create da.make_now
		Result := da.date.compact_date.out
		Result.append("_")
		Result.append(da.time.compact_time.out)
		Result.append("_"+da.time.fractional_second.out)
	ensure
		not_void: Result /= Void
	end

feature -- Basic operations

feature -- Obsolete

feature -- Inapplicable

feature -- Output

	string_To_file (string_to_write, file_name: STRING) is
			-- Write a string out to file_name. 7/23/03
		require
			stw_not_void: string_to_write /= Void
			fn_not_void: file_name /= Void
		local
			store_file: RAW_FILE
		do
			create store_file.make_open_read_append (file_name)
			store_file.put_string (string_to_write)
			store_file.close
		end

feature {NONE} -- Implementation

invariant
	invariant_clause: True -- Your invariant here

end -- class UTILITIES
